#include <stdio.h>
#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include "pad.h"

/*

First example based on the tutorial from Henry S Fortuna...

This is the most basic example...
A Pong clone that only uses black and white boxes.
Move the paddle up and down witht he pad, press Start+Select to end game.

Read the three first lessons from Henry S Fortuna. (http://www.hsfortuna.pwp.blueyonder.co.uk/)
These lessosns are: Getting started, Controllers and Drawing Graphics.

The most interesting part should be the RenderBox function, I commented as much as possible...
This function will write the raw data to the GIF registers.

One important note though...
If you have been programming on the PC (Intel or compatible), you will probably think a word is 16 bits...
Have a look at page 5 in EEUSER_E.PDF, a word is 32 bits and a QuadWord is 32*4=128 bits !!!

Written by Bamse 2004...
Bamse@playstation2-linux.com

*/

#define PAD_MAX 410
#define PAD_MIN 0

// Function prototypes
void RenderBox(int x, int y, int width, int height);

int iPaddle1Pos = 180;		// Paddle in the middle
int iBallXPos = 315;
int iBallYPos = 200;
int iBallXDir = -2;
int iBallYDir = 2;
					   
int main(void) 
{
  	// sps2 handle 
  	int iSPS2Descriptor;

	// Initialise the Pad connected to port 1 of the PS2, we initilize it as analogue so we can read the pressure.
	if(pad_init (PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE ) == 0)
	{
		printf("Unable to initialise the pads\n");
		pad_cleanup (PAD_0);
		exit(0);
	}
  	
  	// Initialize sps2     
  	iSPS2Descriptor = sps2Init();
  	if(iSPS2Descriptor < 0)
  		{
  			printf("sps2Init() failed\n");
  			exit(0);
  		}

  	// Initialise the screen.  
  	sps2UScreenInit(0);
  	
  	do 
  	{

		// Read the pad.
		pad_update (PAD_0);

		// Update paddle posistion for up direction
		// We are doing this three times, that means that the harder you press the faster the paddle go...
		// Slight pressure
		if(pad[0].pressures[PAD_PUP] > 0.3) {
			if (iPaddle1Pos>PAD_MIN) {
				iPaddle1Pos--;
			}
		}
		// Medium pressure
		if(pad[0].pressures[PAD_PUP] > 0.6) {
			if (iPaddle1Pos>PAD_MIN) {
				iPaddle1Pos--;
			}
		}
		// Hard pressure pressure
		if(pad[0].pressures[PAD_PUP] > 0.9) {
			if (iPaddle1Pos>PAD_MIN) {
				iPaddle1Pos--;
			}
		}

		// Update paddle posistion for down direction
		// We are doing this three times, that means that the harder you press the faster the paddle go...
		// Slight pressure
		if(pad[0].pressures[PAD_PDOWN] > 0.3) {
			if (iPaddle1Pos<PAD_MAX) {
				iPaddle1Pos++;
			}
		}
		// Medium pressure
		if(pad[0].pressures[PAD_PDOWN] > 0.6) {
			if (iPaddle1Pos<PAD_MAX) {
				iPaddle1Pos++;
			}
		}
		// Hard pressure pressure
		if(pad[0].pressures[PAD_PDOWN] > 0.9) {
			if (iPaddle1Pos<PAD_MAX) {
				iPaddle1Pos++;
			}
		}

		// Update the ball position...
		iBallXPos += iBallXDir;
		iBallYPos += iBallYDir;

		// Did we bounce against the right wall???
		if (iBallXPos > 620) {
			iBallXDir = -iBallXDir;
		}

		// Did we bounce against the left wall???
		if (iBallXPos < 10) {
			// Where is the paddle compared to the ball???
			int iDiff = iBallYPos - iPaddle1Pos;
			// Missed the paddle.
			if (iDiff < -10 || iDiff > 50) {
				// Reset ball since we missed the paddle...
				iBallYPos = 200;
				iBallXPos = 315;
				iBallXDir = -2;
				iBallYDir = 2;
			} else {
				// We hit the paddle, the ball bounced...
				iBallXDir = -iBallXDir;
			}
		}

		// Did we bounce against the upper wall???
		if (iBallYPos > 460) {
			iBallYDir = -iBallYDir;
		}
		// Did we bounce against the lower wall???
		if (iBallYPos < 10) {
			iBallYDir = -iBallYDir;
		}


    	// Clear the screen buffer to (r,g,b)
    	sps2UScreenClear(0x00,0x00,0x00);

    	// Upper box.
    	RenderBox(0,0,640,10);

    	// Lower box.
    	RenderBox(0,470,640,10);

    	// Divider box.
    	RenderBox(315,0,10,480);

    	// Paddle box.
    	RenderBox(0,10+iPaddle1Pos,10,50);

    	// Ball box.
    	RenderBox(iBallXPos, iBallYPos, 10, 10);

    	// Swap display buffers
    	sps2UScreenSwap();

	// Press start and select together to end the loop	
	}while(!((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)));
	
	// Release the pad device and clean up.
	pad_cleanup (PAD_0);
  
	// Shut down the screen
	sps2UScreenShutdown();

	// Release sps2
	sps2Release(iSPS2Descriptor);
  
  return 0;
}


void RenderBox(int x, int y, int width, int height)
{
	sps2GIFTag_t GIFTag;
  	sps2GIFPackedRegister_t GIFReg;
 
  	// Create the GIFTag  
  	GIFTag.i128=0;       			// Blank it out
  	GIFTag.s.NLOOP=1;    			// 1 entry.  We are rendering one sprite.
  	GIFTag.s.PRE=1;      			// Send our PRIM value to the GIF via this GIFTag
  	GIFTag.s.EOP=1;      			// This is the end of the packets, no more GIFTags are to be expected after the sprite.
  	GIFTag.s.PRIM=6;    			// Sprite, basically just a square
  	GIFTag.s.FLG=GIF_FLG_PACKED;	// Use packed format for data
  	GIFTag.s.NREG=0;     			// Start at 0 and increment as needed
  
  	// Add RGBAQ register.
  	SPS2_SET_GIF_REG(GIFTag, GIFTag.s.NREG++, GIF_REG_RGBAQ);
  	// Add XYZ2 register.
  	SPS2_SET_GIF_REG(GIFTag, GIFTag.s.NREG++, GIF_REG_XYZ2);
  	// Add XYZ2 register.
  	SPS2_SET_GIF_REG(GIFTag, GIFTag.s.NREG++, GIF_REG_XYZ2);

	/*
	Look at the SPS2_SET_GIF_REG Macro...
	The most important thing to understand is that we are doing something
	to the upper 64 bits of the register.
	Basically, the flags provided in uiValue will be set in the correct position (iRegNo) in the
	upper 64 bits (ai64[0] is the lower 64 bits of gifTag and ai64[1] is the upper 64 bits).

	#define SPS2_SET_GIF_REG(gifTag, iRegNo, uiValue) \
	{((sps2GIFTag_t *) &gifTag)->ai64[1] |= ((uiValue & 0xf) << (iRegNo << 2));}
  	*/
		
   	// Send the GIFtag to the GIF FIFO
  	DPUT_EE_GIF_FIFO(GIFTag.i128);

	/*
	See page 20 to 24 in EEUSER_E.PDF.
	I'm not going to dig too deep into this, we are writing to one of the registers.
	You can see that the EE registers reside in 0x1000_0000 to 0x1FFF_FFFF...
	On Page 23 we can see the that the GIF_FIFO register can be found @ 0x1000_6000.
	So basically this Macro will write the GIFTag to address 0x1000_6000.
	The reason the macro is using the SPS2_QPUT function is to make sure all 128 bits get written at once to the register.

	#define DPUT_EE_GIF_FIFO(value)          SPS2_QPUT(0x00016000L,value)
	*/	

	/*
	Let's recap and see what just happenend... See Page 151 in the EEUSER_E.PDF
	We created one GIFTag that tells the GS to draw one sprite (NLOOP=1 with PRIM=6).
	If PRE = 0, you need to send a separate PRIM via a GIFReg. Leave PRE = 1.
	The data we send will be in the GIF_FLG_PACKED format.
	We then set three Registers to describe the upcomming data, one color, and two positions.
	After setting these regs, NREG = 3 and bits 64 to 75 of the GIFTag have been updated.
	Bits 64 to 75 tells the GIF that the following three GIFRegs will be one color and two positions.
	Note that the data MUST be sent in the specified order...
	*/

   	// Prepare and send the RGBAQ register
   	GIFReg.i128=0;
   	GIFReg.RGBAQ.s.R=0xFF;		// Red
   	GIFReg.RGBAQ.s.G=0xFF;		// Green
   	GIFReg.RGBAQ.s.B=0xFF;		// Blue
   	GIFReg.RGBAQ.s.A=0x80;		// Alpha
   	DPUT_EE_GIF_FIFO(GIFReg.i128);

   	// Prepare and send the first XYZ2 register
   	GIFReg.i128=0;
   	GIFReg.XYZ2.s.X=(x + 1728) << 4;	// Explained later.
   	GIFReg.XYZ2.s.Y=(y + 1808) << 4;	// Explained later.
   	GIFReg.XYZ2.s.Z=1;					// Needs to be at least 1 and fit within 24 bits...
   	DPUT_EE_GIF_FIFO(GIFReg.i128);

   	// Prepare and send the second XYZ2 register
   	GIFReg.i128=0;
   	GIFReg.XYZ2.s.X=(x + width + 1728) << 4;
   	GIFReg.XYZ2.s.Y=(y + height + 1808) << 4;
   	GIFReg.XYZ2.s.Z=1;
   	DPUT_EE_GIF_FIFO(GIFReg.i128);
    
	/*
	Wait a minute, what's up with this (x + 1728) << 4 stuff ???
	Have a quick look at Page 155 in the EEUSER_E.PDF...
	X and Y are 16 bits with 4 fractional bits...
	Hopefully that would explain the mysterious << 4, we shift 4 bits to zero out the fractionals...
	So now we have the (x + 1728) part left to explain.
	Now have a quick look at Page 26 in the GSUSER_E.PDF...
	The actual screen is 4095.9375 by 4095.9375 so the middle of the screen is pretty much 2048, 2048.
	However the screen resolution (for VGA) is 640x480.
	So the upper corner of the screen would be 2048-(640/2), 2048-(480/2) or 1728,1808...
	*/

	while (*EE_GIF_STAT & (0x1F << 24));	// Wait for GIF FIFO to empty...

	/*
	We have 16 registers to write to, if we send off more boxes without waiting they might overflow the registers.
	If the registers are full, the upcomming Tags or Registers will be ignored.
	Believe me, this is not a good thing and will probably resulting in a flickering screen... ;-)
	By the way, *EE_GIF_STAT is actually address 0x1000_3020.
	You can find the GIF_STAT register on page 164 in EEUSER_E.PDF.
	*/

}
